<?php
/*----------------------------------------------*\
| Project.: CS:Source Admin (cssourceadmin)
| File....: csadmin.class.php
| Info....: CSAdmin config file
| Author..: "ThorN" thorn@clanbank.com
| Date....: 11-29-2004
|
| Copyright (C) 2004 Michael "ThorN" Thornton
|
| This program is free software; you can 
| redistribute it and/or modify it under the terms
| of the GNU General Public License as published by
| the Free Software Foundation; either version 2 of
| the License, or (at your option) any later version.
|
| This program is distributed in the hope that it
| will be useful, but WITHOUT ANY WARRANTY; without
| even the implied warranty of MERCHANTABILITY or
| FITNESS FOR A PARTICULAR PURPOSE. See the GNU
| General Public License for more details.
|
| You should have received a copy of the GNU General
| Public License along with this program; if not,
| write to the Free Software Foundation, Inc.
| 59 Temple Place, Suite 330
| Boston, MA 02111-1307 USA
| 
|------------------------------------------------|
| Changes.:
|	11-29-2004 "ThorN" thorn@clanbank.com
|		Added &$errors to CSAdmin::Status with 
|		output for parsing errors
|
|	12-27-2004 "ThorN" thorn@clanbank.com
|		Added ::Kick(), ::Ban(), and ::UnBan()
\*----------------------------------------------*/

require_once 'rcon.class.php';

class CSAdmin
{
	var $password;
	var $host;
	var $port;
	var $r;

	function CSAdmin($host, $pass, $port = 27015) 
	{
		$this->password = $pass;
		$this->host = $host;
		$this->port = $port;

		$this->r =& new rcon($this->host, $this->port, $this->password);
		$this->r->Auth();
	} //func

	function Rcon($cmd, $escape = true, $debug = false) 
	{
			return $this->r->rconCommand($cmd, $escape);
	} //func

	function Status(&$errors, &$res) 
	{
		// $res = $this->Rcon('status');
		// call Rcon directly so status works even in debug mode
		$res = $this->r->rconCommand('status');

		if ( !$res ) 
			return false;

		$lines = explode("\n", trim($res));

		$info = array();
		$players = array();
		$header = true;
		$errors = array();
		for ( $i = 0; $i < count($lines); ++$i ) 
		{
			$line = trim($lines[$i]);
		    if ( $header && !$line ) 
		    {
				$header = false;
				++$i;
			}
			elseif ( $header ) 
			{
				list($k, $v) = explode(':', $line, 2);
				$info[trim($k)] = trim($v);
			}
			elseif ( $line[0] == '#' 
				&& preg_match('/^#\s*([0-9]+)\s+"(.*?)"\s*([A-Z0-9_:]+)\s+([0-9:]+)\s+([0-9]+)\s+([0-9]+)\s+([a-z]+)\s+([0-9.:]+)(\s.*)?/i', $line, $regs) )
			{
				$players[$regs[1]] = array(
					'id' => $regs[1],
					'name' => $regs[2],
					'steam' => $regs[3],
					'connected' => $regs[4],
					'ping' => $regs[5],
					'loss' => $regs[6],
					'state' => $regs[7],
					'address' => $regs[8]
				);

				if ( $regs[9] ) 
					$errors[] = 'Invalid data at end of user line: ' . trim($regs[9]);
					
			}
			elseif ( $line[0] == '#' )
			{
				$errors[] = 'Could not parse user line: ' . $line;
			}
			else
			{
				$errors[] = 'Could not identify line: ' . $line;
			}
		}

		return array($info, $players);
	} //func

	function Say($m, $args = null) 
	{
		$numargs = func_num_args();

		if ( $numargs > 1 ) 
		{
			$args = func_get_args();
			$m = array_shift($args);
			$m = vsprintf($m, $args);
		}

		return $this->Rcon(sprintf('"say" "%s"', $m), false);
	} //func

	function Kick($playerId, $dur = 0, $reason = null) 
	{
		if ( $dur > 0 ) 
			$cmd = sprintf('"banid" "%s" "%%s" "kick"', $dur);
		elseif ( $reason ) 
			$cmd = sprintf('"kickid" %%s %s', trim($reason));
		else		
			$cmd = '"kickid" "%s"';

		return $this->Rcon(sprintf($cmd, $playerId), false);
	} //func

	function Ban($playerId) 
	{
		$s = $this->Rcon(sprintf('banid 0 %s kick', $playerId));
		if ( $s ) 
			$this->Rcon('writeid');

		return $s;
	} //func

	function UnBan($playerId) 
	{
		$s = $this->Rcon(sprintf('removeid %s', $playerId));
		if ( $s ) 
			$this->Rcon('writeid');

		return $s;
	} //func
} //class
?>